var profile = "all"
var intervalCounter = 0
var status = "1" // 0: init, 1: ready
var apm // apm chart
var apm_data // apm data
var heatmap // heatmap
var heatmap_data // heatmap data
var bars = {} // progress bars
var lastTime = 0
var researchOrderField = "time"
var colors = [
	"#0000ff",
	"#A01500",
	"#00ff00",
	"#FEFE09",
	"#00ffff",
	"#AA2F9B",
	"#b9b9b9",
	"#FE5609"
]

// reset any elements that accumulate and only have an add action
function resetSections() {
	$(".chat table").html("")
	$(".researches table").html("")
	$("td > strike > span").unwrap()
	bars = {}
}

// runs when status changes from "init" to "ready"
// populates page with all initial content
function populate(data) {
	console.log("Now in realtime mode: "+data.time)
	resetSections()
	lastTime = data.currentTime
	if(active(["all"])) {
		loadMap(data.map)
		addChat(data.chat)
		addResearches(data.researches)
		$(".container").show()
	}
	if(active(["all", "apm"])) {
		setupAPM(data.players)
		addAPM(data.apm)
	}
	if(active(["all", "teams"])) {
		loadPlayers(data.players, data.teamNames)
	}
	console.log("Populate completed")
}

// runs whenever status is "ready"
function update(data) {
	if((data.summary.currentTime == undefined) || (parseInt(data.summary.currentTime) < parseInt(lastTime))) {
		setLoading()
		status = "1"
	}
	lastTime = data.summary.currentTime
	if(active(["all"])) {
		updateTime(data.summary.time)
		addChat(data.chat)
		updateTribute(data.tribute)
		updateTrade(data.trade)
	}
	if(active(["all", "heatmap"])) {
		updateHeatmap(data.heatmap)
	}
	if(active(["all", "researches"])) {
		addResearches(data.researches)
	}
	if(active(["all", "apm"])) {
		addAPM(data.apm)
	}
	if(active(["all", "teams"])) {
		loadTitle(data.summary.title)
		updatePlayers(data.summary.teams)
	}
}

// update player data
function updatePlayers(data) {
	for (var team in data) {
		for (var player_id in data[team]) {
			p = data[team][player_id]
			$(".age"+p.pid).text(p.age)
			if (p.rating > 0) {
				$(".rating"+p.pid).text(p.rating)
			}
			if(p.resigned) {
				$("td > span.player"+p.pid).wrap("<strike>")
			}
		}
	}
}

// <div class='progress-bar progress'></div>
// add researches
function addResearches(data) {
	for (var i in data) {
		var value = data[i]
		var name = "prog"+value.tech_id+value.player_id+value.time
		var tname = "name"+value.tech_id+value.player_id+value.time
		var rowId = "row"+value.tech_id+value.player_id+value.time
		if($("#" + name).length == 0) {
			var row = "<tr id='"+rowId+"' sort='"+value[researchOrderField]+"'>"
			row += "<td>"+value.time_f+"</td><td>-</td><td>"+value.time_ff+"</td><td align='right'><span class='p"+value.player_id+"'>"+value.player_name+"</span></td>"
			row += "<td align='right'><span id='"+tname+"'>"+value.technology+"</span></td><td class='progress'><div class='p"+value.player_id+" progress progress-wrap' id='"+name+"'></div></td>"
			row += "</tr>"
			var added = false
			if(value.percent < 1.0) {
				$(".researches table tr").each(function(index) {
					if(value[researchOrderField] > $(this).attr("sort")) {
						$(this).before(row)
						added = true
						return false
					}
				});
			}
			if(added == false) {
				$(".researches table").prepend(row)
			}
			bars[name] = new ProgressBar.Line("#"+name, {})
		}
		line = bars[name]
		if(value.percent >= 1.0) {
			if(active(["all"])) {
				$("#"+name).hide()
			}
			if(active(["researches"])) {
				$("#"+rowId).hide()
			}
		} else {
			line.animate(value.percent, {duration: 1000})
		}
		if(value.canceled) {
			$("td > span#"+tname).wrap("<strike>")
		}
	}
}

// update tribute
function updateTribute(data) {
	var html = "None"
	html = ""
	for (var t in data) {
		
		html += "<table><tr><td></td>"
		for (var p in data[t].to) {
			html += "<td><span class='p"+data[t].to[p]["color"]+"'>"+data[t].to[p]["player"]+"</span></td>"
		}
		for (var p in data[t].from) {
			html += "<tr><td><span class='p"+data[t].from[p]["player"]["color"]+"'>"+data[t].from[p]["player"]["player"]+"</span></td>"
			for (var i in data[t].from[p]["row"]) {
				resources = data[t].from[p]["row"][i]
				html += "<td align='right' class='border'>"+tributeResources(resources)+"</td>"
			}
			html += "</tr>"
		}
		html += "</tr>"
		html += "</table>"
	}
	$(".tribute .tributeInner").html(html)
}

function tributeResources(data) {
	var html = "<table width='100%'>"
	if (data.length > 0) {
		html += "<tr><td align='right'><img src='/static/img/food.png'/></td><td width='50%'>"+data[0]+"</td>"
		html += "<td align='right'><img src='/static/img/wood.png'/></td><td width='50%'>"+data[1]+"</td></tr>"
		html += "<tr><td align='right'><img src='/static/img/stone.png'/></td><td width='50%'>"+data[2]+"</td>"
		html += "<td align='right'><img src='/static/img/gold.png'/></td><td width='50%'>"+data[3]+"</td></tr>"
	}
	return html += "</table>"
}

// update trade
function updateTrade(data) {
	var html = "None"
	html = "<table><tr><td></td><td colspan='2' align='center' class='vborder'><img src='/static/img/food.png'/></td>"
	html += "<td colspan='2' align='center' class='vborder'><img src='/static/img/wood.png'/></td><td colspan='2' align='center'><img src='/static/img/stone.png'/></td></tr>"
	html += "<tr><td></td><td class='vborder'>Spent</td><td class='vborder'>Sold</td><td class='vborder'>Spent</td><td class='vborder'>Sold</td><td class='vborder'>Spent</td><td>Sold</td></tr>"
	for (var p in data) {
		html += "<tr><td align='right'><span class='p"+data[p]["color"]+"'>"+data[p]["player"]+"</span></td>"
		for (var r in data[p]["trade"]) {
			html += "<td align='right' class='border'>"+data[p]["trade"][r]["spent"]+"</td><td align='right' class='border'>"+data[p]["trade"][r]["sold"]+"</td>"
		}
		html += "</tr>"
	}
	html += "</table>"
	$(".trade .tradeInner").html(html)
}

// update game time
function updateTime(data) {
	$(".currentTime").html(data)
}

// set up the heatmap
function setupHeatmap() {
	heatmap = h337.create({
		container: document.querySelector('.heatmapC'),
		radius: "25"
	})
	heatmap_data = []
}

// update heatmap
function updateHeatmap(data) {
	for (var p1 in data) {
		var add = true
		for (var p2 in heatmap_data) {
			if(data[p1].x == heatmap_data[p2].x && data[p1].y == heatmap_data[p2].y) {
				add = false
				if(heatmap_data[p2].fresh == false) {
					heatmap_data[p2].fresh == true
				}
			}
		}
		if (add) {
			heatmap_data.push(data[p1])
		}
	}
}

// animate heatmap
function animateHeatmap(data) {
	var newData = []
	for (var point in heatmap_data) {
		if(heatmap_data[point].value > 3) {
			heatmap_data[point].fresh = false
		}
		if(heatmap_data[point].fresh) {
			heatmap_data[point].value += 0.1
		} else {
			heatmap_data[point].value -= 0.1
		}
		if(heatmap_data[point].value > 0) {
			newData.push(heatmap_data[point])
		}
	}
	heatmap_data = newData
	heatmap.setData({max: 3, data: heatmap_data})
}

// add apm
function addAPM(data) {
	for(var i in data) {
		set = data[i]
		for(var player in set.apm) {
			point = data[i].apm[player]
			var next = {x: set.time, y: point}
			var last = apm_data[player][apm_data[player].length - 1]
			if(next.x > last.x) {
				apm_data[player].push(next)
				//console.log(next)
			}
			if(apm_max_seconds > 0 && apm_data[player].length > apm_max_seconds) {
				apm_data[player].shift()				
			}
		}
	}
	apm.render()
}

// add chat
function addChat(data) {
	var html = ""
	for (var i in data) {
		line = data[i]
		html = "<tr><td>"+line.view+"</td><td>"+line.time+"</td><td align='right'><span class='p"+line.player_id+"'>"+line.player_name+"</span>:</td><td>"+line.message+"</td></tr>" + html
	}
	$(".chat table").prepend(html)
}

// load minimap
function loadMap(data) {
	$(".map").html(data)}

// load title
function loadTitle(data) {
	$(".title").html(data)
}

// load players
function loadPlayers(teams, teamNames) {
	var html = ""
	for (var team in teams) {
		html += "<div class='team'>"
		if(teams[team].length > 1) {
			html += "<h2>"+teamNames[team]+"</h2>"
		}
		html += "<table>"
		for (var player_id in teams[team]) {
			p = teams[team][player_id]
			cl = "player"+p.pid
			html += "<tr><td><span class='p"+p.color+" "+cl+"'>"+p.name+"</span></td><td>"+p.civ_name+"</td><td class='age"+p.pid+"'>"+p.age+"</td>"
			html += "<td class='rating"+p.pid+"'></td>"
			html += "</tr>"
		}
		html += "</table></div>"
	}
	$(".teams").html(html)
}

function setLoading() {
	$(".title").text("Loading ...")
	$(".currentTime").text("")
	$(".container").hide()
	console.log("Loading ...")
}

// once a second
function tickSec() {
	var lastStatus = status
	requestStatus(setStatus)
	if(status == "0" && lastStatus != "0") {
		requestPopulate(populate)
	} else if(status == "0") {
		requestState(update)
	} else {
		setLoading()
	}
}

// can't reach server - set status to "init"
function ajaxError() {
	setStatus("0")
}

// set global status
function setStatus(data) {
	status = data
}

// request status (init or ready)
function requestStatus(callback) {
	$.ajax({url: "/status", success: callback, error: ajaxError, async: false})
}

// request game state
function requestState(callback) {
	$.ajax({url: "/state/"+profile, success: callback, error: ajaxError})
}

// request initial populate data
function requestPopulate(callback) {
	$.ajax({url: "/populate/"+profile, success: callback, error: ajaxError, async: false})
}

// 100 ms tick interval
function tick(interval) {
	intervalCounter++
	if(intervalCounter == 10) {
		intervalCounter = 0
		tickSec()
	}
	if(active(["all", "heatmap"])) {
		animateHeatmap()
	}
	setTimeout( function() { tick(interval) }, interval)
}

// set up apm chart
function setupAPM(teams) {
	$("#apm").empty()
	apm_data = []
	series = []
	// add an initial point for every possible player
	for (var i=0;i<8;i++) {
		apm_data.push([{x:0,y:0}])
	}
	// add series for players
	for (var team in teams) {
		for (var player_id in teams[team]) {
			p = teams[team][player_id]
			series.push({
				type: "line",
				markerSize: 0,
				lineThickness: 3,
				color: colors[p.color-1],
				dataPoints: apm_data[p.pid]
			})
		}
	}
	// initialize chart
	apm = new CanvasJS.Chart("apm",{
		data: series,
		backgroundColor: null,
		axisX:{        
	        gridThickness: 0,
	        labelFontColor: "white",
	        lineColor: "black",
	        lineThickness: 1,
	        labelFontFamily: "Georgia",
	        valueFormatString: " ",
      		tickLength: 0
	      },
	      axisY:{        
	       gridThickness: 0,
	       labelFontColor: "white",
	       lineColor: "black",
	       lineThickness: 1,
	       labelFontFamily: "Georgia"
	      },
	      toolTip:{
	      	enabled: false
	      }
	});
	apm.render()
}

function active(list) {
	return list.indexOf(profile) >= 0
}

function updatePosition(elem) {
	var id = elem.attr("id");
	var left = localStorage.getItem(id+"left");
	var top = localStorage.getItem(id+"top");
	var hide = localStorage.getItem(id+"hide");
	if(hide != null && hide) {
		elem.hide()
	}
	if(left != null && top != null) {
		elem.css({ left: left + "px", top: top + "px" , position: "absolute"});
	}
}

// set up configurable interface
function setupConfigurable() {
	$(".draggable").each(function(){
		updatePosition($(this))
	})
	$(".draggable").draggable({ grid: [ 10, 10 ], stop: function () {
		$(".draggable").each(function() {
			var left = $(this).position().left
			var top = $(this).position().top
			var id = $(this).attr("id")
			localStorage.setItem(id+"left", left)
			localStorage.setItem(id+"top", top)
		})
	}
	}).dblclick(function(){
		$(this).hide()
		var id = $(this).attr("id")
		localStorage.setItem(id+"hide", true)
	})
}

// entry point
function setupAnalysis(prof) {
	profile = prof
	if(research_order == "finish") {
		researchOrderField = "time_e"
	} else {
		researchOrderField = "time"
	}
	if(active(["all", "heatmap"])) {
		setupHeatmap()
	}
	jQuery.fx.interval = 100
	setupConfigurable()
	tick(100)
}